@@ -44,7 +44,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <vector>
 
 #include "nr_socket_proxy_config.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
 
@@ -57,7 +56,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "nsCOMPtr.h"
 #include "nsComponentManagerUtils.h"
 #include "nsError.h"
-#include "nsIEventTarget.h"
 #include "nsNetCID.h"
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
@@ -65,6 +63,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "runnable_utils.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
+#include "nsIUUIDGenerator.h"
 
 // nICEr includes
 extern "C" {
@@ -97,6 +96,10 @@ extern "C" {
 #include "rlogconnector.h"
 #include "test_nr_socket.h"
 
+extern "C" {
+#include "mdns_service/mdns_service.h"
+}
+
 namespace mozilla {
 
 using std::shared_ptr;
@@ -252,8 +255,8 @@ nsresult NrIceTurnServer::ToNicerTurnStruct(nr_ice_turn_server* server) const {
   // C++03 23.2.4, Paragraph 1 stipulates that the elements
   // in std::vector must be contiguous, and can therefore be
   // used as input to functions expecting C arrays.
-  int r = r_data_create(&server->password, const_cast<UCHAR*>(&password_[0]),
-                        password_.size());
+  const UCHAR* data = password_.empty() ? nullptr : &password_[0];
+  int r = r_data_create(&server->password, data, password_.size());
   if (r) {
     RFREE(server->username);
     return NS_ERROR_OUT_OF_MEMORY;
@@ -266,7 +269,6 @@ NrIceCtx::NrIceCtx(const std::string& name, Policy policy)
     : connection_state_(ICE_CTX_INIT),
       gathering_state_(ICE_CTX_GATHER_INIT),
       name_(name),
-      offerer_(false),
       ice_controlling_set_(false),
       streams_(),
       ctx_(nullptr),
@@ -276,7 +278,8 @@ NrIceCtx::NrIceCtx(const std::string& name, Policy policy)
       trickle_(true),
       policy_(policy),
       nat_(nullptr),
-      proxy_config_(nullptr) {}
+      proxy_config_(nullptr),
+      obfuscate_host_addresses_(false) {}
 
 /* static */
 RefPtr<NrIceCtx> NrIceCtx::Create(const std::string& name, bool allow_loopback,
@@ -315,6 +318,10 @@ void NrIceCtx::DestroyStream(const std::string& id) {
     streams_.erase(it);
     preexisting_stream->Close();
   }
+
+  if (streams_.empty()) {
+    SetGatheringState(ICE_CTX_GATHER_INIT);
+  }
 }
 
 // Handler callbacks
@@ -428,21 +435,26 @@ void NrIceCtx::trickle_cb(void* arg, nr_ice_ctx* ice_ctx,
   }
 
   if (!candidate) {
-    s->SignalCandidate(s, "", stream->ufrag);
+    s->SignalCandidate(s, "", stream->ufrag, "", "");
     return;
   }
 
+  std::string actual_addr;
+  std::string mdns_addr;
+  ctx->GenerateObfuscatedAddress(candidate, &mdns_addr, &actual_addr);
+
   // Format the candidate.
   char candidate_str[NR_ICE_MAX_ATTRIBUTE_SIZE];
   int r = nr_ice_format_candidate_attribute(candidate, candidate_str,
-                                            sizeof(candidate_str));
+                                            sizeof(candidate_str),
+                                            ctx->obfuscate_host_addresses_);
   MOZ_ASSERT(!r);
   if (r) return;
 
   MOZ_MTLOG(ML_INFO, "NrIceCtx(" << ctx->name_ << "): trickling candidate "
                                  << candidate_str);
 
-  s->SignalCandidate(s, candidate_str, stream->ufrag);
+  s->SignalCandidate(s, candidate_str, stream->ufrag, mdns_addr, actual_addr);
 }
 
 void NrIceCtx::InitializeGlobals(bool allow_loopback, bool tcp_enabled,
@@ -519,6 +531,12 @@ void NrIceCtx::InitializeGlobals(bool allow_loopback, bool tcp_enabled,
   }
 }
 
+void NrIceCtx::SetTargetForDefaultLocalAddressLookup(
+    const std::string& target_ip, uint16_t target_port) {
+  nr_ice_set_target_for_default_local_address_lookup(ctx_, target_ip.c_str(),
+                                                     target_port);
+}
+
 #define MAXADDRS 100  // mirrors setting in ice_ctx.c
 
 /* static */
@@ -729,40 +747,6 @@ NrIceStats NrIceCtx::Destroy() {
     stats.turn_438s = ctx_->stats.turn_438s;
   }
 
-  if (!ice_start_time_.IsNull()) {
-    TimeDuration time_delta = TimeStamp::Now() - ice_start_time_;
-    ice_start_time_ = TimeStamp();  // null out
-
-    if (offerer_) {
-      Telemetry::Accumulate(Telemetry::WEBRTC_ICE_OFFERER_ABORT_TIME,
-                            time_delta.ToMilliseconds());
-    } else {
-      Telemetry::Accumulate(Telemetry::WEBRTC_ICE_ANSWERER_ABORT_TIME,
-                            time_delta.ToMilliseconds());
-    }
-
-    unsigned char rate_limit_bit_pattern = 0;
-    if (!mozilla::nr_socket_short_term_violation_time().IsNull() &&
-        mozilla::nr_socket_short_term_violation_time() >= ice_start_time_) {
-      rate_limit_bit_pattern |= 1;
-    }
-    if (!mozilla::nr_socket_long_term_violation_time().IsNull() &&
-        mozilla::nr_socket_long_term_violation_time() >= ice_start_time_) {
-      rate_limit_bit_pattern |= 2;
-    }
-
-    if (connection_state_ == ICE_CTX_FAILED) {
-      Telemetry::Accumulate(
-          Telemetry::WEBRTC_STUN_RATE_LIMIT_EXCEEDED_BY_TYPE_GIVEN_FAILURE,
-          rate_limit_bit_pattern);
-    } else if (connection_state_ == ICE_CTX_CONNECTED ||
-               connection_state_ == ICE_CTX_COMPLETED) {
-      Telemetry::Accumulate(
-          Telemetry::WEBRTC_STUN_RATE_LIMIT_EXCEEDED_BY_TYPE_GIVEN_SUCCESS,
-          rate_limit_bit_pattern);
-    }
-  }
-
   if (peer_) {
     nr_ice_peer_ctx_destroy(&peer_);
   }
@@ -781,7 +765,7 @@ NrIceStats NrIceCtx::Destroy() {
   return stats;
 }
 
-NrIceCtx::~NrIceCtx() {}
+NrIceCtx::~NrIceCtx() = default;
 
 void NrIceCtx::destroy_peer_ctx() { nr_ice_peer_ctx_destroy(&peer_); }
 
@@ -866,12 +850,12 @@ nsresult NrIceCtx::SetResolver(nr_resolver* resolver) {
   return NS_OK;
 }
 
-nsresult NrIceCtx::SetProxyServer(NrSocketProxyConfig&& config) {
+nsresult NrIceCtx::SetProxyConfig(NrSocketProxyConfig&& config) {
   proxy_config_.reset(new NrSocketProxyConfig(std::move(config)));
   return NS_OK;
 }
 
-void NrIceCtx::SetCtxFlags(bool default_route_only, bool proxy_only) {
+void NrIceCtx::SetCtxFlags(bool default_route_only) {
   ASSERT_ON_THREAD(sts_target_);
 
   if (default_route_only) {
@@ -879,21 +863,16 @@ void NrIceCtx::SetCtxFlags(bool default_route_only, bool proxy_only) {
   } else {
     nr_ice_ctx_remove_flags(ctx_, NR_ICE_CTX_FLAGS_ONLY_DEFAULT_ADDRS);
   }
-
-  if (proxy_only) {
-    nr_ice_ctx_add_flags(ctx_, NR_ICE_CTX_FLAGS_ONLY_PROXY);
-  } else {
-    nr_ice_ctx_remove_flags(ctx_, NR_ICE_CTX_FLAGS_ONLY_PROXY);
-  }
 }
 
-nsresult NrIceCtx::StartGathering(bool default_route_only, bool proxy_only) {
+nsresult NrIceCtx::StartGathering(bool default_route_only,
+                                  bool obfuscate_host_addresses) {
   ASSERT_ON_THREAD(sts_target_);
-  SetGatheringState(ICE_CTX_GATHER_STARTED);
 
-  SetCtxFlags(default_route_only, proxy_only);
+  obfuscate_host_addresses_ = obfuscate_host_addresses;
+
+  SetCtxFlags(default_route_only);
 
-  TimeStamp start = TimeStamp::Now();
   // This might start gathering for the first time, or again after
   // renegotiation, or might do nothing at all if gathering has already
   // finished.
@@ -901,18 +880,14 @@ nsresult NrIceCtx::StartGathering(bool default_route_only, bool proxy_only) {
 
   if (!r) {
     SetGatheringState(ICE_CTX_GATHER_COMPLETE);
-    Telemetry::AccumulateTimeDelta(
-        Telemetry::WEBRTC_ICE_NR_ICE_GATHER_TIME_IMMEDIATE_SUCCESS, start);
-  } else if (r != R_WOULDBLOCK) {
+  } else if (r == R_WOULDBLOCK) {
+    SetGatheringState(ICE_CTX_GATHER_STARTED);
+  } else {
+    SetGatheringState(ICE_CTX_GATHER_COMPLETE);
     MOZ_MTLOG(ML_ERROR, "ICE FAILED: Couldn't gather ICE candidates for '"
                             << name_ << "', error=" << r);
     SetConnectionState(ICE_CTX_FAILED);
-    Telemetry::AccumulateTimeDelta(
-        Telemetry::WEBRTC_ICE_NR_ICE_GATHER_TIME_IMMEDIATE_FAILURE, start);
     return NS_ERROR_FAILURE;
-  } else {
-    Telemetry::AccumulateTimeDelta(Telemetry::WEBRTC_ICE_NR_ICE_GATHER_TIME,
-                                   start);
   }
 
   return NS_OK;
@@ -977,16 +952,13 @@ bool NrIceCtx::HasStreamsToConnect() const {
   return false;
 }
 
-nsresult NrIceCtx::StartChecks(bool offerer) {
+nsresult NrIceCtx::StartChecks() {
   int r;
   if (!HasStreamsToConnect()) {
     // Nothing to do
     return NS_OK;
   }
 
-  offerer_ = offerer;
-  ice_start_time_ = TimeStamp::Now();
-
   r = nr_ice_peer_ctx_pair_candidates(peer_);
   if (r) {
     MOZ_MTLOG(ML_ERROR, "ICE FAILED: Couldn't pair candidates on " << name_);
@@ -1044,44 +1016,6 @@ void NrIceCtx::UpdateNetworkState(bool online) {
 void NrIceCtx::SetConnectionState(ConnectionState state) {
   if (state == connection_state_) return;
 
-  if (!ice_start_time_.IsNull() && (state > ICE_CTX_CHECKING)) {
-    TimeDuration time_delta = TimeStamp::Now() - ice_start_time_;
-    ice_start_time_ = TimeStamp();
-
-    switch (state) {
-      case ICE_CTX_INIT:
-      case ICE_CTX_CHECKING:
-        MOZ_CRASH();
-        break;
-      case ICE_CTX_CONNECTED:
-      case ICE_CTX_COMPLETED:
-        if (offerer_) {
-          Telemetry::Accumulate(Telemetry::WEBRTC_ICE_OFFERER_SUCCESS_TIME,
-                                time_delta.ToMilliseconds());
-        } else {
-          Telemetry::Accumulate(Telemetry::WEBRTC_ICE_ANSWERER_SUCCESS_TIME,
-                                time_delta.ToMilliseconds());
-        }
-        break;
-      case ICE_CTX_FAILED:
-        if (offerer_) {
-          Telemetry::Accumulate(Telemetry::WEBRTC_ICE_OFFERER_FAILURE_TIME,
-                                time_delta.ToMilliseconds());
-        } else {
-          Telemetry::Accumulate(Telemetry::WEBRTC_ICE_ANSWERER_FAILURE_TIME,
-                                time_delta.ToMilliseconds());
-        }
-        break;
-      case ICE_CTX_DISCONNECTED:
-        // We get this every time an ICE disconnect gets reported.
-        // Do we want a Telemetry probe counting how often this happens?
-        break;
-      case ICE_CTX_CLOSED:
-        // This doesn't seem to be used...
-        break;
-    }
-  }
-
   MOZ_MTLOG(ML_INFO, "NrIceCtx(" << name_ << "): state " << connection_state_
                                  << "->" << state);
   connection_state_ = state;
@@ -1109,6 +1043,54 @@ void NrIceCtx::SetGatheringState(GatheringState state) {
   SignalGatheringStateChange(this, state);
 }
 
+void NrIceCtx::GenerateObfuscatedAddress(nr_ice_candidate* candidate,
+                                         std::string* mdns_address,
+                                         std::string* actual_address) {
+  if (candidate->type == HOST && obfuscate_host_addresses_) {
+    char addr[64];
+    if (nr_transport_addr_get_addrstring(&candidate->addr, addr,
+                                         sizeof(addr))) {
+      return;
+    }
+
+    *actual_address = addr;
+
+    const auto& iter = obfuscated_host_addresses_.find(*actual_address);
+    if (iter != obfuscated_host_addresses_.end()) {
+      *mdns_address = iter->second;
+    } else {
+      nsresult rv;
+      nsCOMPtr<nsIUUIDGenerator> uuidgen =
+          do_GetService("@mozilla.org/uuid-generator;1", &rv);
+      // If this fails, we'll return a zero UUID rather than something
+      // unexpected.
+      nsID id = {};
+      id.Clear();
+      if (NS_SUCCEEDED(rv)) {
+        rv = uuidgen->GenerateUUIDInPlace(&id);
+        if (NS_FAILED(rv)) {
+          id.Clear();
+        }
+      }
+
+      char chars[NSID_LENGTH];
+      id.ToProvidedString(chars);
+      // The string will look like {64888863-a253-424a-9b30-1ed285d20142},
+      // we want to trim off the braces.
+      const char* ptr_to_id = chars;
+      ++ptr_to_id;
+      chars[NSID_LENGTH - 2] = 0;
+
+      std::ostringstream o;
+      o << ptr_to_id << ".local";
+      *mdns_address = o.str();
+
+      obfuscated_host_addresses_[*actual_address] = *mdns_address;
+    }
+    candidate->mdns_addr = r_strdup(mdns_address->c_str());
+  }
+}
+
 }  // namespace mozilla
 
 // Reimplement nr_ice_compute_codeword to avoid copyright issues
