@@ -7,7 +7,6 @@
 #include "signaling/src/jsep/JsepTrackEncoding.h"
 
 #include <algorithm>
-#include <iostream>
 
 namespace mozilla {
 void JsepTrack::GetNegotiatedPayloadTypes(
@@ -16,7 +15,7 @@ void JsepTrack::GetNegotiatedPayloadTypes(
     return;
   }
 
-  for (const auto* encoding : mNegotiatedDetails->mEncodings.values) {
+  for (const auto& encoding : mNegotiatedDetails->mEncodings) {
     GetPayloadTypes(encoding->GetCodecs(), payloadTypes);
   }
 
@@ -28,9 +27,9 @@ void JsepTrack::GetNegotiatedPayloadTypes(
 
 /* static */
 void JsepTrack::GetPayloadTypes(
-    const std::vector<JsepCodecDescription*>& codecs,
+    const std::vector<UniquePtr<JsepCodecDescription>>& codecs,
     std::vector<uint16_t>* payloadTypes) {
-  for (JsepCodecDescription* codec : codecs) {
+  for (const auto& codec : codecs) {
     uint16_t pt;
     if (!codec->GetPtAsInt(&pt)) {
       MOZ_ASSERT(false);
@@ -41,10 +40,10 @@ void JsepTrack::GetPayloadTypes(
 }
 
 void JsepTrack::EnsureNoDuplicatePayloadTypes(
-    std::vector<JsepCodecDescription*>* codecs) {
+    std::vector<UniquePtr<JsepCodecDescription>>* codecs) {
   std::set<uint16_t> uniquePayloadTypes;
 
-  for (JsepCodecDescription* codec : *codecs) {
+  for (auto& codec : *codecs) {
     // We assume there are no dupes in negotiated codecs; unnegotiated codecs
     // need to change if there is a clash.
     if (!codec->mEnabled ||
@@ -83,38 +82,44 @@ void JsepTrack::EnsureNoDuplicatePayloadTypes(
   }
 }
 
-void JsepTrack::EnsureSsrcs(SsrcGenerator& ssrcGenerator) {
-  if (mSsrcs.empty()) {
-    uint32_t ssrc;
-    if (!ssrcGenerator.GenerateSsrc(&ssrc)) {
+void JsepTrack::EnsureSsrcs(SsrcGenerator& ssrcGenerator, size_t aNumber) {
+  while (mSsrcs.size() < aNumber) {
+    uint32_t ssrc, rtxSsrc;
+    if (!ssrcGenerator.GenerateSsrc(&ssrc) ||
+        !ssrcGenerator.GenerateSsrc(&rtxSsrc)) {
       return;
     }
     mSsrcs.push_back(ssrc);
+    mSsrcToRtxSsrc[ssrc] = rtxSsrc;
+    MOZ_ASSERT(mSsrcs.size() == mSsrcToRtxSsrc.size());
   }
 }
 
 void JsepTrack::PopulateCodecs(
-    const std::vector<JsepCodecDescription*>& prototype) {
-  for (const JsepCodecDescription* prototypeCodec : prototype) {
+    const std::vector<UniquePtr<JsepCodecDescription>>& prototype) {
+  mPrototypeCodecs.clear();
+  for (const auto& prototypeCodec : prototype) {
     if (prototypeCodec->mType == mType) {
-      mPrototypeCodecs.values.push_back(prototypeCodec->Clone());
-      mPrototypeCodecs.values.back()->mDirection = mDirection;
+      mPrototypeCodecs.emplace_back(prototypeCodec->Clone());
+      mPrototypeCodecs.back()->mDirection = mDirection;
     }
   }
 
-  EnsureNoDuplicatePayloadTypes(&mPrototypeCodecs.values);
+  EnsureNoDuplicatePayloadTypes(&mPrototypeCodecs);
 }
 
 void JsepTrack::AddToOffer(SsrcGenerator& ssrcGenerator,
                            SdpMediaSection* offer) {
-  AddToMsection(mPrototypeCodecs.values, offer);
+  AddToMsection(mPrototypeCodecs, offer);
 
   if (mDirection == sdp::kSend) {
     std::vector<JsConstraints> constraints;
     if (offer->IsSending()) {
       constraints = mJsEncodeConstraints;
     }
-    AddToMsection(constraints, sdp::kSend, ssrcGenerator, offer);
+
+    AddToMsection(constraints, sdp::kSend, ssrcGenerator,
+                  IsRtxEnabled(mPrototypeCodecs), offer);
   }
 }
 
@@ -123,38 +128,58 @@ void JsepTrack::AddToAnswer(const SdpMediaSection& offer,
                             SdpMediaSection* answer) {
   // We do not modify mPrototypeCodecs here, since we're only creating an
   // answer. Once offer/answer concludes, we will update mPrototypeCodecs.
-  PtrVector<JsepCodecDescription> codecs;
-  codecs.values = GetCodecClones();
-  NegotiateCodecs(offer, &codecs.values);
-  if (codecs.values.empty()) {
+  std::vector<UniquePtr<JsepCodecDescription>> codecs =
+      NegotiateCodecs(offer, true);
+  if (codecs.empty()) {
     return;
   }
 
-  AddToMsection(codecs.values, answer);
+  AddToMsection(codecs, answer);
 
   if (mDirection == sdp::kSend) {
     std::vector<JsConstraints> constraints;
     if (answer->IsSending()) {
       constraints = mJsEncodeConstraints;
-      std::vector<SdpRidAttributeList::Rid> rids;
+      std::vector<std::pair<SdpRidAttributeList::Rid, bool>> rids;
       GetRids(offer, sdp::kRecv, &rids);
       NegotiateRids(rids, &constraints);
     }
-    AddToMsection(constraints, sdp::kSend, ssrcGenerator, answer);
+
+    AddToMsection(constraints, sdp::kSend, ssrcGenerator, IsRtxEnabled(codecs),
+                  answer);
   }
 }
 
-void JsepTrack::SetJsConstraints(
+bool JsepTrack::SetJsConstraints(
     const std::vector<JsConstraints>& constraintsList) {
+  bool constraintsChanged = mJsEncodeConstraints != constraintsList;
   mJsEncodeConstraints = constraintsList;
+
+  // Also update negotiated details with constraints, as these can change
+  // without negotiation.
+
+  if (!mNegotiatedDetails) {
+    return constraintsChanged;
+  }
+
+  for (auto& encoding : mNegotiatedDetails->mEncodings) {
+    for (const JsConstraints& jsConstraints : mJsEncodeConstraints) {
+      if (jsConstraints.rid == encoding->mRid) {
+        encoding->mConstraints = jsConstraints.constraints;
+      }
+    }
+  }
+
+  return constraintsChanged;
 }
 
-void JsepTrack::AddToMsection(const std::vector<JsepCodecDescription*>& codecs,
-                              SdpMediaSection* msection) {
+void JsepTrack::AddToMsection(
+    const std::vector<UniquePtr<JsepCodecDescription>>& codecs,
+    SdpMediaSection* msection) {
   MOZ_ASSERT(msection->GetMediaType() == mType);
   MOZ_ASSERT(!codecs.empty());
 
-  for (const JsepCodecDescription* codec : codecs) {
+  for (const auto& codec : codecs) {
     codec->AddToMediaSection(*msection);
   }
 
@@ -165,9 +190,6 @@ void JsepTrack::AddToMsection(const std::vector<JsepCodecDescription*>& codecs,
     } else {
       for (const std::string& streamId : mStreamIds) {
         msection->AddMsid(streamId, mTrackId);
-        // TODO(bug 1402912) Interop hack; older Firefox barfs if there is more
-        // than one msid. Remove when safe.
-        break;
       }
     }
   }
@@ -176,14 +198,15 @@ void JsepTrack::AddToMsection(const std::vector<JsepCodecDescription*>& codecs,
 // Updates the |id| values in |constraintsList| with the rid values in |rids|,
 // where necessary.
 void JsepTrack::NegotiateRids(
-    const std::vector<SdpRidAttributeList::Rid>& rids,
+    const std::vector<std::pair<SdpRidAttributeList::Rid, bool>>& rids,
     std::vector<JsConstraints>* constraintsList) const {
-  for (const SdpRidAttributeList::Rid& rid : rids) {
-    if (!FindConstraints(rid.id, *constraintsList)) {
+  for (const auto& ridAndPaused : rids) {
+    if (!FindConstraints(ridAndPaused.first.id, *constraintsList)) {
       // Pair up the first JsConstraints with an empty id, if it exists.
       JsConstraints* constraints = FindConstraints("", *constraintsList);
       if (constraints) {
-        constraints->rid = rid.id;
+        constraints->rid = ridAndPaused.first.id;
+        constraints->paused = ridAndPaused.second;
       }
     }
   }
@@ -197,22 +220,44 @@ void JsepTrack::UpdateSsrcs(SsrcGenerator& ssrcGenerator, size_t encodings) {
   // Right now, the spec does not permit changing the number of encodings after
   // the initial creation of the sender, so we don't need to worry about things
   // like a new encoding inserted in between two pre-existing encodings.
-  while (mSsrcs.size() < numSsrcs) {
-    uint32_t ssrc;
-    if (!ssrcGenerator.GenerateSsrc(&ssrc)) {
-      return;
+  EnsureSsrcs(ssrcGenerator, numSsrcs);
+  PruneSsrcs(numSsrcs);
+
+  MOZ_ASSERT(!mSsrcs.empty());
+}
+
+void JsepTrack::PruneSsrcs(size_t aNumSsrcs) {
+  mSsrcs.resize(aNumSsrcs);
+
+  // We might have duplicate entries in mSsrcs, so we need to resize first and
+  // then remove ummatched rtx ssrcs.
+  auto itor = mSsrcToRtxSsrc.begin();
+  while (itor != mSsrcToRtxSsrc.end()) {
+    if (std::find(mSsrcs.begin(), mSsrcs.end(), itor->first) == mSsrcs.end()) {
+      itor = mSsrcToRtxSsrc.erase(itor);
+    } else {
+      ++itor;
+    }
+  }
+}
+
+bool JsepTrack::IsRtxEnabled(
+    const std::vector<UniquePtr<JsepCodecDescription>>& codecs) const {
+  for (const auto& codec : codecs) {
+    if (codec->mType == SdpMediaSection::kVideo &&
+        static_cast<const JsepVideoCodecDescription*>(codec.get())
+            ->mRtxEnabled) {
+      return true;
     }
-    mSsrcs.push_back(ssrc);
   }
 
-  mSsrcs.resize(numSsrcs);
-  MOZ_ASSERT(!mSsrcs.empty());
+  return false;
 }
 
 void JsepTrack::AddToMsection(const std::vector<JsConstraints>& constraintsList,
                               sdp::Direction direction,
                               SsrcGenerator& ssrcGenerator,
-                              SdpMediaSection* msection) {
+                              bool requireRtxSsrcs, SdpMediaSection* msection) {
   UniquePtr<SdpSimulcastAttribute> simulcast(new SdpSimulcastAttribute);
   UniquePtr<SdpRidAttributeList> rids(new SdpRidAttributeList);
   for (const JsConstraints& constraints : constraintsList) {
@@ -223,7 +268,8 @@ void JsepTrack::AddToMsection(const std::vector<JsConstraints>& constraintsList,
       rids->mRids.push_back(rid);
 
       SdpSimulcastAttribute::Version version;
-      version.choices.push_back(constraints.rid);
+      version.choices.push_back(
+          SdpSimulcastAttribute::Encoding(constraints.rid, false));
       if (direction == sdp::kSend) {
         simulcast->sendVersions.push_back(version);
       } else {
@@ -239,13 +285,27 @@ void JsepTrack::AddToMsection(const std::vector<JsConstraints>& constraintsList,
 
   if (mType != SdpMediaSection::kApplication && mDirection == sdp::kSend) {
     UpdateSsrcs(ssrcGenerator, constraintsList.size());
-    msection->SetSsrcs(mSsrcs, mCNAME);
+
+    if (requireRtxSsrcs) {
+      MOZ_ASSERT(mSsrcs.size() == mSsrcToRtxSsrc.size());
+      std::vector<uint32_t> allSsrcs;
+      UniquePtr<SdpSsrcGroupAttributeList> group(new SdpSsrcGroupAttributeList);
+      for (const auto& [ssrc, rtxSsrc] : mSsrcToRtxSsrc) {
+        allSsrcs.push_back(ssrc);
+        allSsrcs.push_back(rtxSsrc);
+        group->PushEntry(SdpSsrcGroupAttributeList::kFid, {ssrc, rtxSsrc});
+      }
+      msection->SetSsrcs(allSsrcs, mCNAME);
+      msection->GetAttributeList().SetAttribute(group.release());
+    } else {
+      msection->SetSsrcs(mSsrcs, mCNAME);
+    }
   }
 }
 
-void JsepTrack::GetRids(const SdpMediaSection& msection,
-                        sdp::Direction direction,
-                        std::vector<SdpRidAttributeList::Rid>* rids) const {
+void JsepTrack::GetRids(
+    const SdpMediaSection& msection, sdp::Direction direction,
+    std::vector<std::pair<SdpRidAttributeList::Rid, bool>>* rids) const {
   rids->clear();
   if (!msection.GetAttributeList().HasAttribute(
           SdpAttribute::kSimulcastAttribute)) {
@@ -269,15 +329,11 @@ void JsepTrack::GetRids(const SdpMediaSection& msection,
     return;
   }
 
-  if (versions->type != SdpSimulcastAttribute::Versions::kRid) {
-    // No support for PT-based simulcast, yet.
-    return;
-  }
-
   for (const SdpSimulcastAttribute::Version& version : *versions) {
     if (!version.choices.empty()) {
       // We validate that rids are present (and sane) elsewhere.
-      rids->push_back(*msection.FindRid(version.choices[0]));
+      rids->push_back(std::make_pair(*msection.FindRid(version.choices[0].rid),
+                                     version.choices[0].paused));
     }
   }
 }
@@ -294,18 +350,28 @@ JsepTrack::JsConstraints* JsepTrack::FindConstraints(
 
 void JsepTrack::CreateEncodings(
     const SdpMediaSection& remote,
-    const std::vector<JsepCodecDescription*>& negotiatedCodecs,
+    const std::vector<UniquePtr<JsepCodecDescription>>& negotiatedCodecs,
     JsepTrackNegotiatedDetails* negotiatedDetails) {
   negotiatedDetails->mTias = remote.GetBandwidth("TIAS");
+
+  webrtc::RtcpMode rtcpMode = webrtc::RtcpMode::kCompound;
+  // rtcp-rsize (video only)
+  if (remote.GetMediaType() == SdpMediaSection::kVideo &&
+      remote.GetAttributeList().HasAttribute(
+          SdpAttribute::kRtcpRsizeAttribute)) {
+    rtcpMode = webrtc::RtcpMode::kReducedSize;
+  }
+  negotiatedDetails->mRtpRtcpConf = RtpRtcpConfig(rtcpMode);
+
   // TODO add support for b=AS if TIAS is not set (bug 976521)
 
-  std::vector<SdpRidAttributeList::Rid> rids;
+  std::vector<std::pair<SdpRidAttributeList::Rid, bool>> rids;
   GetRids(remote, sdp::kRecv, &rids);  // Get rids we will send
   NegotiateRids(rids, &mJsEncodeConstraints);
   if (rids.empty()) {
     // Add dummy value with an empty id to make sure we get a single unicast
     // stream.
-    rids.push_back(SdpRidAttributeList::Rid());
+    rids.push_back(std::make_pair(SdpRidAttributeList::Rid(), false));
   }
 
   size_t max_streams = 1;
@@ -316,89 +382,115 @@ void JsepTrack::CreateEncodings(
   // Drop SSRCs if less RIDs were offered than we have encoding constraints
   // Just in case.
   if (mSsrcs.size() > max_streams) {
-    mSsrcs.resize(max_streams);
+    PruneSsrcs(max_streams);
   }
 
   // For each stream make sure we have an encoding, and configure
   // that encoding appropriately.
   for (size_t i = 0; i < max_streams; ++i) {
-    if (i == negotiatedDetails->mEncodings.values.size()) {
-      negotiatedDetails->mEncodings.values.push_back(new JsepTrackEncoding);
+    if (i == negotiatedDetails->mEncodings.size()) {
+      negotiatedDetails->mEncodings.emplace_back(new JsepTrackEncoding);
     }
 
-    JsepTrackEncoding* encoding = negotiatedDetails->mEncodings.values[i];
+    auto& encoding = negotiatedDetails->mEncodings[i];
 
-    for (const JsepCodecDescription* codec : negotiatedCodecs) {
-      if (rids[i].HasFormat(codec->mDefaultPt)) {
+    for (const auto& codec : negotiatedCodecs) {
+      if (rids[i].first.HasFormat(codec->mDefaultPt)) {
         encoding->AddCodec(*codec);
       }
     }
 
-    encoding->mRid = rids[i].id;
+    encoding->mRid = rids[i].first.id;
+    encoding->mPaused = rids[i].second;
     // If we end up supporting params for rid, we would handle that here.
 
     // Incorporate the corresponding JS encoding constraints, if they exist
     for (const JsConstraints& jsConstraints : mJsEncodeConstraints) {
-      if (jsConstraints.rid == rids[i].id) {
+      if (jsConstraints.rid == rids[i].first.id) {
         encoding->mConstraints = jsConstraints.constraints;
       }
     }
   }
 }
 
-std::vector<JsepCodecDescription*> JsepTrack::GetCodecClones() const {
-  std::vector<JsepCodecDescription*> clones;
-  for (const JsepCodecDescription* codec : mPrototypeCodecs.values) {
-    clones.push_back(codec->Clone());
+std::vector<UniquePtr<JsepCodecDescription>> JsepTrack::GetCodecClones() const {
+  std::vector<UniquePtr<JsepCodecDescription>> clones;
+  for (const auto& codec : mPrototypeCodecs) {
+    clones.emplace_back(codec->Clone());
   }
   return clones;
 }
 
-static bool CompareCodec(const JsepCodecDescription* lhs,
-                         const JsepCodecDescription* rhs) {
+static bool CompareCodec(const UniquePtr<JsepCodecDescription>& lhs,
+                         const UniquePtr<JsepCodecDescription>& rhs) {
   return lhs->mStronglyPreferred && !rhs->mStronglyPreferred;
 }
 
-void JsepTrack::NegotiateCodecs(
-    const SdpMediaSection& remote, std::vector<JsepCodecDescription*>* codecs,
-    std::map<std::string, std::string>* formatChanges) const {
-  MOZ_ASSERT(codecs->size());
-  PtrVector<JsepCodecDescription> unnegotiatedCodecs;
-  std::swap(unnegotiatedCodecs.values, *codecs);
+std::vector<UniquePtr<JsepCodecDescription>> JsepTrack::NegotiateCodecs(
+    const SdpMediaSection& remote, bool isOffer) {
+  std::vector<UniquePtr<JsepCodecDescription>> negotiatedCodecs;
+  std::vector<UniquePtr<JsepCodecDescription>> newPrototypeCodecs;
 
   // Outer loop establishes the remote side's preference
   for (const std::string& fmt : remote.GetFormats()) {
-    for (size_t i = 0; i < unnegotiatedCodecs.values.size(); ++i) {
-      JsepCodecDescription* codec = unnegotiatedCodecs.values[i];
+    for (auto& codec : mPrototypeCodecs) {
       if (!codec || !codec->mEnabled || !codec->Matches(fmt, remote)) {
         continue;
       }
 
-      std::string originalFormat = codec->mDefaultPt;
-      if (codec->Negotiate(fmt, remote)) {
-        codecs->push_back(codec);
-        unnegotiatedCodecs.values[i] = nullptr;
-        if (formatChanges) {
-          (*formatChanges)[originalFormat] = codec->mDefaultPt;
+      // First codec of ours that matches. See if we can negotiate it.
+      UniquePtr<JsepCodecDescription> clone(codec->Clone());
+      if (clone->Negotiate(fmt, remote, isOffer)) {
+        // If negotiation changed the payload type, remember that for later
+        codec->mDefaultPt = clone->mDefaultPt;
+
+        // Remember whether we negotiated rtx and the associated pt for later.
+        if (codec->mType == SdpMediaSection::kVideo) {
+          JsepVideoCodecDescription* videoCodec =
+              static_cast<JsepVideoCodecDescription*>(codec.get());
+          JsepVideoCodecDescription* cloneVideoCodec =
+              static_cast<JsepVideoCodecDescription*>(clone.get());
+          bool useRtx =
+              Preferences::GetBool("media.peerconnection.video.use_rtx", false);
+          videoCodec->mRtxEnabled = useRtx && cloneVideoCodec->mRtxEnabled;
+          videoCodec->mRtxPayloadType = cloneVideoCodec->mRtxPayloadType;
         }
+
+        // Moves the codec out of mPrototypeCodecs, leaving an empty
+        // UniquePtr, so we don't use it again. Also causes successfully
+        // negotiated codecs to be placed up front in the future.
+        newPrototypeCodecs.emplace_back(std::move(codec));
+        negotiatedCodecs.emplace_back(std::move(clone));
         break;
       }
     }
   }
 
+  // newPrototypeCodecs contains just the negotiated stuff so far. Add the rest.
+  for (auto& codec : mPrototypeCodecs) {
+    if (codec) {
+      newPrototypeCodecs.emplace_back(std::move(codec));
+    }
+  }
+
+  // Negotiated stuff is up front, so it will take precedence when ensuring
+  // there are no duplicate payload types.
+  EnsureNoDuplicatePayloadTypes(&newPrototypeCodecs);
+  std::swap(newPrototypeCodecs, mPrototypeCodecs);
+
   // Find the (potential) red codec and ulpfec codec or telephone-event
   JsepVideoCodecDescription* red = nullptr;
   JsepVideoCodecDescription* ulpfec = nullptr;
   JsepAudioCodecDescription* dtmf = nullptr;
   // We can safely cast here since JsepTrack has a MediaType and only codecs
   // that match that MediaType (kAudio or kVideo) are added.
-  for (auto codec : *codecs) {
+  for (auto& codec : negotiatedCodecs) {
     if (codec->mName == "red") {
-      red = static_cast<JsepVideoCodecDescription*>(codec);
+      red = static_cast<JsepVideoCodecDescription*>(codec.get());
     } else if (codec->mName == "ulpfec") {
-      ulpfec = static_cast<JsepVideoCodecDescription*>(codec);
+      ulpfec = static_cast<JsepVideoCodecDescription*>(codec.get());
     } else if (codec->mName == "telephone-event") {
-      dtmf = static_cast<JsepAudioCodecDescription*>(codec);
+      dtmf = static_cast<JsepAudioCodecDescription*>(codec.get());
     }
   }
   // if we have a red codec remove redundant encodings that don't exist
@@ -410,7 +502,7 @@ void JsepTrack::NegotiateCodecs(
     std::swap(unnegotiatedEncodings, red->mRedundantEncodings);
     for (auto redundantPt : unnegotiatedEncodings) {
       std::string pt = std::to_string(redundantPt);
-      for (auto codec : *codecs) {
+      for (const auto& codec : negotiatedCodecs) {
         if (pt == codec->mDefaultPt) {
           red->mRedundantEncodings.push_back(redundantPt);
           break;
@@ -423,10 +515,10 @@ void JsepTrack::NegotiateCodecs(
   // a rtcpfb attr). If we see both red and ulpfec codecs, we enable FEC
   // on all the other codecs.
   if (red && ulpfec) {
-    for (auto codec : *codecs) {
+    for (auto& codec : negotiatedCodecs) {
       if (codec->mName != "red" && codec->mName != "ulpfec") {
         JsepVideoCodecDescription* videoCodec =
-            static_cast<JsepVideoCodecDescription*>(codec);
+            static_cast<JsepVideoCodecDescription*>(codec.get());
         videoCodec->EnableFec(red->mDefaultPt, ulpfec->mDefaultPt);
       }
     }
@@ -437,69 +529,42 @@ void JsepTrack::NegotiateCodecs(
   // rtcpfb attr). If we see the telephone-event codec, we enabled dtmf
   // support on all the other audio codecs.
   if (dtmf) {
-    for (auto codec : *codecs) {
+    for (auto& codec : negotiatedCodecs) {
       JsepAudioCodecDescription* audioCodec =
-          static_cast<JsepAudioCodecDescription*>(codec);
+          static_cast<JsepAudioCodecDescription*>(codec.get());
       audioCodec->mDtmfEnabled = true;
     }
   }
 
   // Make sure strongly preferred codecs are up front, overriding the remote
   // side's preference.
-  std::stable_sort(codecs->begin(), codecs->end(), CompareCodec);
+  std::stable_sort(negotiatedCodecs.begin(), negotiatedCodecs.end(),
+                   CompareCodec);
 
-  // TODO(bug 814227): Remove this once we're ready to put multiple codecs in an
-  // answer.  For now, remove all but the first codec unless the red codec
-  // exists, in which case we include the others per RFC 5109, section 14.2.
-  if (!codecs->empty() && !red) {
-    std::vector<JsepCodecDescription*> codecsToKeep;
-
-    bool foundPreferredCodec = false;
-    for (auto codec : *codecs) {
-      if (codec == dtmf) {
-        codecsToKeep.push_back(codec);
-        // TODO: keep ulpfec when we enable it in Bug 875922
-        // } else if (codec == ulpfec) {
-        //   codecsToKeep.push_back(codec);
-      } else if (!foundPreferredCodec) {
-        codecsToKeep.insert(codecsToKeep.begin(), codec);
-        foundPreferredCodec = true;
-      } else {
-        delete codec;
-      }
-    }
-
-    *codecs = codecsToKeep;
+  if (!red) {
+    // No red, remove ulpfec
+    negotiatedCodecs.erase(
+        std::remove_if(negotiatedCodecs.begin(), negotiatedCodecs.end(),
+                       [ulpfec](const UniquePtr<JsepCodecDescription>& codec) {
+                         return codec.get() == ulpfec;
+                       }),
+        negotiatedCodecs.end());
+    // Make sure there's no dangling ptr here
+    ulpfec = nullptr;
   }
+
+  return negotiatedCodecs;
 }
 
 void JsepTrack::Negotiate(const SdpMediaSection& answer,
                           const SdpMediaSection& remote) {
-  PtrVector<JsepCodecDescription> negotiatedCodecs;
-  negotiatedCodecs.values = GetCodecClones();
-
-  std::map<std::string, std::string> formatChanges;
-  NegotiateCodecs(remote, &negotiatedCodecs.values, &formatChanges);
-
-  // Use |formatChanges| to update mPrototypeCodecs
-  size_t insertPos = 0;
-  for (size_t i = 0; i < mPrototypeCodecs.values.size(); ++i) {
-    if (formatChanges.count(mPrototypeCodecs.values[i]->mDefaultPt)) {
-      // Update the payload type to what was negotiated
-      mPrototypeCodecs.values[i]->mDefaultPt =
-          formatChanges[mPrototypeCodecs.values[i]->mDefaultPt];
-      // Move this negotiated codec up front
-      std::swap(mPrototypeCodecs.values[insertPos], mPrototypeCodecs.values[i]);
-      ++insertPos;
-    }
-  }
-
-  EnsureNoDuplicatePayloadTypes(&mPrototypeCodecs.values);
+  std::vector<UniquePtr<JsepCodecDescription>> negotiatedCodecs =
+      NegotiateCodecs(remote, &answer != &remote);
 
   UniquePtr<JsepTrackNegotiatedDetails> negotiatedDetails =
       MakeUnique<JsepTrackNegotiatedDetails>();
 
-  CreateEncodings(remote, negotiatedCodecs.values, negotiatedDetails.get());
+  CreateEncodings(remote, negotiatedCodecs, negotiatedDetails.get());
 
   if (answer.GetAttributeList().HasAttribute(SdpAttribute::kExtmapAttribute)) {
     for (auto& extmapAttr : answer.GetAttributeList().GetExtmap().mExtmaps) {
@@ -515,7 +580,7 @@ void JsepTrack::Negotiate(const SdpMediaSection& answer,
     }
   }
 
-  mNegotiatedDetails = Move(negotiatedDetails);
+  mNegotiatedDetails = std::move(negotiatedDetails);
 }
 
 // When doing bundle, if all else fails we can try to figure out which m-line a
