@@ -6,11 +6,72 @@
 
 from __future__ import absolute_import, print_function, unicode_literals
 
+import copy
+from re import search
+
+import six
 from taskgraph import try_option_syntax
+from taskgraph.parameters import Parameters
 from taskgraph.util.attributes import match_run_on_projects, match_run_on_hg_branches
 
 _target_task_methods = {}
 
+# Some tasks show up in the target task set, but are possibly special cases,
+# uncommon tasks, or tasks running against limited hardware set that they
+# should only be selectable with --full.
+UNCOMMON_TRY_TASK_LABELS = [
+    # Platforms and/or Build types
+    r'build-.*-gcp',  # Bug 1631990
+    r'build-.*-aarch64',  # Bug 1631990
+    r'mingwclang',  # Bug 1631990
+    r'valgrind',  # Bug 1631990
+    # Android tasks
+    r'android-geckoview-docs',
+    r'android-hw',
+    # Windows tasks
+    r'windows10-64-ref-hw',
+    r'windows10-aarch64',
+    # Linux tasks
+    r'linux-',  # hide all linux32 tasks by default - bug 1599197
+    r'linux1804-32',  # hide linux32 tests - bug 1599197
+    r'linux.*web-platform-tests.*-fis-',  # hide wpt linux fission tests - bug 1610879
+    # Test tasks
+    r'web-platform-tests.*backlog',  # hide wpt jobs that are not implemented yet - bug 1572820
+    r'-ccov/',
+    # Shippable build tests, except those that don't have opt versions - bug 1638014
+    # blacklist tasks on these platforms that aren't part of the named test suites,
+    # which are known to only run on shippable builds
+    r'(linux1804-64|windows10-64|windows7-32)-shippable(?!.*(awsy|browsertime|marionette-headless|raptor|talos|web-platform-tests-wdspec-headless))',  # noqa - too long
+]
+
+
+# These are live site performance tests we run three times a week
+LIVE_SITES = [
+    "amazon-search",
+    "bbc",
+    "booking-sf",
+    "cnn-ampstories",
+    "discord",
+    "espn",
+    "expedia",
+    "facebook-cristiano",
+    "fashionbeans",
+    "google",
+    "google-accounts",
+    "imdb-firefox",
+    "jianshu",
+    "medium-article",
+    "microsoft-support",
+    "nytimes",
+    "people-article",
+    "reddit-thread",
+    "rumble-fox",
+    "stackoverflow-question",
+    "urbandictionary-define",
+    "wikia-marvel",
+    "youtube-watch"
+]
+
 
 def _target_task(name):
     def wrap(func):
@@ -24,13 +85,18 @@ def get_method(method):
     return _target_task_methods[method]
 
 
-def filter_out_nightly(task, parameters):
+def filter_out_shipping_phase(task, parameters):
     return (
+        # nightly still here because of geckodriver
         not task.attributes.get('nightly') and
         task.attributes.get('shipping_phase') in (None, 'build')
         )
 
 
+def filter_out_devedition(task, parameters):
+    return not task.attributes.get('shipping_product') == 'devedition'
+
+
 def filter_out_cron(task, parameters):
     """
     Filter out tasks that run via cron.
@@ -57,6 +123,26 @@ def filter_on_platforms(task, platforms):
     return (platform in platforms)
 
 
+def filter_by_uncommon_try_tasks(task, optional_filters=None):
+    """Filters tasks based on blacklist rules.
+
+    Args:
+        task (str): String representing the task name.
+        optional_filters (list, optional):
+            Additional filters to apply to task filtering.
+
+    Returns:
+        (Boolean): True if task does not match any known filters.
+            False otherwise.
+    """
+    filters = UNCOMMON_TRY_TASK_LABELS
+    if optional_filters:
+        filters = copy.deepcopy(filters)
+        filters.extend(optional_filters)
+
+    return not any(search(pattern, task) for pattern in UNCOMMON_TRY_TASK_LABELS)
+
+
 def filter_release_tasks(task, parameters):
     platform = task.attributes.get('build_platform')
     if platform in (
@@ -109,12 +195,11 @@ def _try_option_syntax(full_task_graph, parameters, graph_config):
     """Generate a list of target tasks based on try syntax in
     parameters['message'] and, for context, the full task graph."""
     options = try_option_syntax.TryOptionSyntax(parameters, full_task_graph, graph_config)
-    target_tasks_labels = [t.label for t in full_task_graph.tasks.itervalues()
-                           if options.task_matches(t)]
+    target_tasks_labels = [t.label for t in six.itervalues(full_task_graph.tasks)
+                           if options.task_matches(t) and filter_by_uncommon_try_tasks(t.label)]
 
     attributes = {
         k: getattr(options, k) for k in [
-            'env',
             'no_retry',
             'tag',
         ]
@@ -130,17 +215,14 @@ def _try_option_syntax(full_task_graph, parameters, graph_config):
         # If the developer wants test jobs to be rebuilt N times we add that value here
         if options.trigger_tests > 1 and 'unittest_suite' in task.attributes:
             task.attributes['task_duplicates'] = options.trigger_tests
-            task.attributes['profile'] = False
 
         # If the developer wants test talos jobs to be rebuilt N times we add that value here
         if options.talos_trigger_tests > 1 and task.attributes.get('unittest_suite') == 'talos':
             task.attributes['task_duplicates'] = options.talos_trigger_tests
-            task.attributes['profile'] = options.profile
 
         # If the developer wants test raptor jobs to be rebuilt N times we add that value here
         if options.raptor_trigger_tests > 1 and task.attributes.get('unittest_suite') == 'raptor':
             task.attributes['task_duplicates'] = options.raptor_trigger_tests
-            task.attributes['profile'] = options.profile
 
         task.attributes.update(attributes)
 
@@ -171,50 +253,30 @@ def target_tasks_try(full_task_graph, parameters, graph_config):
         return []
 
 
+@_target_task('try_auto')
+def target_tasks_try_auto(full_task_graph, parameters, graph_config):
+    """Target the tasks which have indicated they should be run on autoland
+    (rather than try) via the `run_on_projects` attributes.
+
+    Should do the same thing as the `default` target tasks method.
+    """
+    params = dict(parameters)
+    params['project'] = 'autoland'
+    parameters = Parameters(**params)
+    return [l for l, t in six.iteritems(full_task_graph.tasks)
+            if standard_filter(t, parameters)
+            and filter_out_shipping_phase(t, parameters)
+            and filter_by_uncommon_try_tasks(t.label)]
+
+
 @_target_task('default')
 def target_tasks_default(full_task_graph, parameters, graph_config):
     """Target the tasks which have indicated they should be run on this project
     via the `run_on_projects` attributes."""
-    return [l for l, t in full_task_graph.tasks.iteritems()
+    return [l for l, t in six.iteritems(full_task_graph.tasks)
             if standard_filter(t, parameters)
-            and filter_out_nightly(t, parameters)]
-
-
-@_target_task('ash_tasks')
-def target_tasks_ash(full_task_graph, parameters, graph_config):
-    """Target tasks that only run on the ash branch."""
-    def filter(task):
-        platform = task.attributes.get('build_platform')
-        # Early return if platform is None
-        if not platform:
-            return False
-        # Only on Linux platforms
-        if 'linux' not in platform:
-            return False
-        # No random non-build jobs either. This is being purposely done as a
-        # blacklist so newly-added jobs aren't missed by default.
-        for p in ('nightly', 'haz', 'artifact', 'cov', 'add-on'):
-            if p in platform:
-                return False
-        for k in ('toolchain', 'l10n'):
-            if k in task.attributes['kind']:
-                return False
-        # and none of this linux64-asan/debug stuff
-        if platform == 'linux64-asan' and task.attributes['build_type'] == 'debug':
-            return False
-        # no non-e10s tests
-        if task.attributes.get('unittest_suite'):
-            if not task.attributes.get('e10s'):
-                return False
-        # don't upload symbols
-        if task.attributes['kind'] == 'upload-symbols':
-            return False
-        return True
-
-    return [l for l, t in full_task_graph.tasks.iteritems()
-            if filter(t)
-            and standard_filter(t, parameters)
-            and filter_out_nightly(t, parameters)]
+            and filter_out_shipping_phase(t, parameters)
+            and filter_out_devedition(t, parameters)]
 
 
 @_target_task('graphics_tasks')
@@ -244,7 +306,7 @@ def target_tasks_valgrind(full_task_graph, parameters, graph_config):
             return True
         return False
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('mozilla_beta_tasks')
@@ -253,7 +315,7 @@ def target_tasks_mozilla_beta(full_task_graph, parameters, graph_config):
     of desktop, plus android CI. The candidates build process involves a pipeline
     of builds and signing, but does not include beetmover or balrog jobs."""
 
-    return [l for l, t in full_task_graph.tasks.iteritems()
+    return [l for l, t in six.iteritems(full_task_graph.tasks)
             if filter_release_tasks(t, parameters)
             and standard_filter(t, parameters)]
 
@@ -264,36 +326,11 @@ def target_tasks_mozilla_release(full_task_graph, parameters, graph_config):
     of desktop, plus android CI. The candidates build process involves a pipeline
     of builds and signing, but does not include beetmover or balrog jobs."""
 
-    return [l for l, t in full_task_graph.tasks.iteritems()
+    return [l for l, t in six.iteritems(full_task_graph.tasks)
             if filter_release_tasks(t, parameters)
             and standard_filter(t, parameters)]
 
 
-@_target_task('mozilla_esr60_tasks')
-def target_tasks_mozilla_esr60(full_task_graph, parameters, graph_config):
-    """Select the set of tasks required for a promotable beta or release build
-    of desktop. The candidates build process involves a pipeline of builds and
-    signing, but does not include beetmover or balrog jobs."""
-
-    def filter(task):
-        if not filter_release_tasks(task, parameters):
-            return False
-
-        if not standard_filter(task, parameters):
-            return False
-
-        platform = task.attributes.get('build_platform')
-
-        # Android is not built on esr60.
-        if platform and 'android' in platform:
-            return False
-
-        # All else was already filtered
-        return True
-
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
-
-
 @_target_task('mozilla_esr68_tasks')
 def target_tasks_mozilla_esr68(full_task_graph, parameters, graph_config):
     """Select the set of tasks required for a promotable beta or release build
@@ -317,7 +354,37 @@ def target_tasks_mozilla_esr68(full_task_graph, parameters, graph_config):
 
         return True
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('mozilla_esr78_tasks')
+def target_tasks_mozilla_esr78(full_task_graph, parameters, graph_config):
+    """Select the set of tasks required for a promotable beta or release build
+    of desktop, plus android CI. The candidates build process involves a pipeline
+    of builds and signing, but does not include beetmover or balrog jobs."""
+
+    def filter(task):
+        if not filter_release_tasks(task, parameters):
+            return False
+
+        if not standard_filter(task, parameters):
+            return False
+
+        platform = task.attributes.get('build_platform')
+
+        # Android is not built on esr78.
+        if platform and 'android' in platform:
+            return False
+
+        test_platform = task.attributes.get('test_platform')
+
+        # Don't run QuantumRender tests on esr78.
+        if test_platform and '-qr/' in test_platform:
+            return False
+
+        return True
+
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('promote_desktop')
@@ -341,7 +408,15 @@ def target_tasks_promote_desktop(full_task_graph, parameters, graph_config):
         if task.attributes.get('shipping_phase') == 'promote':
             return True
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+def is_geckoview(task, parameters):
+    return (
+        task.attributes.get('shipping_product') == 'fennec' and
+        task.kind in ('beetmover-geckoview', 'upload-symbols') and
+        parameters['release_product'] == 'firefox'
+    )
 
 
 @_target_task('push_desktop')
@@ -358,11 +433,16 @@ def target_tasks_push_desktop(full_task_graph, parameters, graph_config):
         # Include promotion tasks; these will be optimized out
         if task.label in filtered_for_candidates:
             return True
+        # XXX: Bug 1612540 - include beetmover jobs for publishing geckoview, along
+        # with the regular Firefox (not Devedition!) releases so that they are at sync
+        if is_geckoview(task, parameters):
+            return True
+
         if task.attributes.get('shipping_product') == parameters['release_product'] and \
                 task.attributes.get('shipping_phase') == 'push':
             return True
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('ship_desktop')
@@ -388,80 +468,21 @@ def target_tasks_ship_desktop(full_task_graph, parameters, graph_config):
         # Include promotion tasks; these will be optimized out
         if task.label in filtered_for_candidates:
             return True
+
+        # XXX: Bug 1619603 - geckoview also ships alongside Firefox RC
+        if is_geckoview(task, parameters) and is_rc:
+            return True
+
         if task.attributes.get('shipping_product') != parameters['release_product'] or \
                 task.attributes.get('shipping_phase') != 'ship':
             return False
 
         if 'secondary' in task.kind:
-                return is_rc
+            return is_rc
         else:
-                return not is_rc
+            return not is_rc
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
-
-
-def _get_filter_promote_fennec(fennec_release_type):
-    def filter_(task):
-        attr = task.attributes.get
-        # Don't ship single locale fennec anymore - Bug 1408083
-        if attr("locale") or attr("chunk_locales"):
-            return False
-
-        return attr('shipping_product') == 'fennec' and \
-            attr('shipping_phase') == 'promote' and \
-            attr('release-type') in (None, fennec_release_type)
-
-    return filter_
-
-
-def _get_filter_ship_fennec(fennec_release_type, filtered_for_candidates, parameters):
-    is_rc = (parameters.get('release_type') == 'esr68-rc')
-
-    def filter_(task):
-        # XXX Starting 68, Geckoview is shipped in a separate target_tasks
-        if task.kind == 'beetmover-geckoview':
-            return False
-
-        # Include candidates build tasks; these will be optimized out
-        if task.label in filtered_for_candidates:
-            return True
-
-        attr = task.attributes.get
-        if attr('shipping_product') != 'fennec' or \
-                attr('shipping_phase') not in ('ship', 'push'):
-            return False
-
-        if attr('release-type') in (None, fennec_release_type):
-            if is_rc:
-                return task.kind in ('release-secondary-notify-ship', 'push-apk')
-            else:
-                return task.kind != 'release-secondary-notify-ship'
-
-    return filter_
-
-
-@_target_task('promote_fennec_release')
-def target_tasks_promote_fennec_release(full_task_graph, parameters, graph_config):
-    """Select the set of tasks required for a candidates build of fennec. The
-    release build process involves a pipeline of builds, signing,
-    and, eventually, uploading the tasks to balrog."""
-
-    filter = _get_filter_promote_fennec(fennec_release_type='release')
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(full_task_graph[l])]
-
-
-@_target_task('ship_fennec_release')
-def target_tasks_ship_fennec_release(full_task_graph, parameters, graph_config):
-    """Select the set of tasks required to ship fennec.
-    Previous build deps will be optimized out via action task."""
-    filter = _get_filter_ship_fennec(
-        fennec_release_type='release',
-        filtered_for_candidates=target_tasks_promote_fennec_release(
-            full_task_graph, parameters, graph_config,
-        ),
-        parameters=parameters,
-    )
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(full_task_graph[l])]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('pine_tasks')
@@ -475,29 +496,180 @@ def target_tasks_pine(full_task_graph, parameters, graph_config):
         # disable asan
         if platform == 'linux64-asan':
             return False
-        # disable non-pine and nightly tasks
-        if standard_filter(task, parameters) or filter_out_nightly(task, parameters):
+        # disable non-pine and tasks with a shipping phase
+        if standard_filter(task, parameters) or filter_out_shipping_phase(task, parameters):
             return True
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
-@_target_task('ship_geckoview_beta')
-def target_tasks_nightly_geckoview(full_task_graph, parameters, graph_config):
-    """Select the set of tasks required to ship geckoview beta. The
-    build process involves a pipeline of builds and an upload to
-    maven.mozilla.org."""
-
+@_target_task('kaios_tasks')
+def target_tasks_kaios(full_task_graph, parameters, graph_config):
+    """The set of tasks to run for kaios integration"""
     def filter(task):
-        # XXX Starting 69, we don't ship Fennec Nightly anymore. We just want geckoview and
-        # its symbols to be uploaded
+        # We disable everything in central, and adjust downstream.
+        return False
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('ship_geckoview')
+def target_tasks_ship_geckoview(full_task_graph, parameters, graph_config):
+    """Select the set of tasks required to ship geckoview nightly. The
+    nightly build process involves a pipeline of builds and an upload to
+    maven.mozilla.org."""
+    def filter(task):
+        # XXX Starting 69, we don't ship Fennec Nightly anymore. We just want geckoview to be
+        # uploaded
         return (
-            task.attributes.get('release-type') == 'beta' and
-            # XXX The shippable geckoview beta are flagged as "nightly"
-            task.attributes.get('nightly') is True and
+            task.attributes.get('shipping_product') == 'fennec' and
             task.kind in ('beetmover-geckoview', 'upload-symbols')
         )
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('fennec_v68')
+def target_tasks_fennec_v68(full_task_graph, parameters, graph_config):
+    """
+    Select tasks required for running tp6m fennec v68 tests
+    """
+    def filter(task):
+        platform = task.attributes.get('build_platform')
+        test_platform = task.attributes.get('test_platform')
+        attributes = task.attributes
+
+        if platform and 'android' not in platform:
+            return False
+        if attributes.get('unittest_suite') != 'raptor':
+            return False
+        if '-fennec68-' in attributes.get('raptor_try_name'):
+            if '-p2' in test_platform and '-arm7' in test_platform:
+                return False
+            if '-youtube-playback-' in attributes.get('raptor_try_name') \
+                    and 'opt' in test_platform:
+                return False
+            return True
+
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task("live_site_perf_testing")
+def target_tasks_live_site_perf_testing(full_task_graph, parameters, graph_config):
+    """
+    Select browsertime live site tasks that should only run once a week.
+    """
+    def filter(task):
+        platform = task.attributes.get('build_platform')
+        attributes = task.attributes
+        vismet = attributes.get('kind') == 'visual-metrics-dep'
+        if attributes.get('unittest_suite') != 'raptor' and not vismet:
+            return False
+        try_name = attributes.get('raptor_try_name')
+        if vismet:
+            platform = task.task.get('extra').get('treeherder-platform')
+            try_name = task.label
+
+        if 'android' not in platform:
+            return False
+        if 'fenix' not in try_name:
+            return False
+        if ('browsertime' not in try_name or
+            'shippable' not in platform or
+            'live' not in try_name):
+            return False
+        for test in LIVE_SITES:
+            if try_name.endswith(test) or try_name.endswith(test + "-e10s"):
+                # These tests run 3 times a week, ignore them
+                return False
+
+        return True
+
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('general_perf_testing')
+def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config):
+    """
+    Select tasks required for running performance tests 3 times a week.
+    """
+    def filter(task):
+        platform = task.attributes.get('build_platform')
+        attributes = task.attributes
+        vismet = attributes.get('kind') == 'visual-metrics-dep'
+        if attributes.get('unittest_suite') != 'raptor' and not vismet:
+            return False
+
+        try_name = attributes.get('raptor_try_name')
+        if vismet:
+            # Visual metric tasks are configured a bit differently
+            platform = task.task.get('extra').get('treeherder-platform')
+            try_name = task.label
+
+        def _run_live_site():
+            for test in LIVE_SITES:
+                if try_name.endswith(test) or try_name.endswith(test + "-e10s"):
+                    return True
+            return False
+
+        # Run chrome and chromium on all platforms available
+        if '-chrome' in try_name:
+            if 'android' in platform:
+                # Run only on shippable android builds
+                if 'shippable' in platform:
+                    if '-live' in try_name:
+                        return _run_live_site()
+                else:
+                    return False
+            else:
+                # Run on all desktop builds
+                return True
+        if '-chromium' in try_name:
+            return True
+
+        # Run raptor scn-power-idle and speedometer for fenix and fennec68
+        if 'shippable' in platform:
+            if 'raptor-scn-power-idle' in try_name \
+                    and ('-fenix' in try_name or '-fennec68' in try_name):
+                return True
+            if 'raptor-speedometer' in try_name \
+                    and '-fennec68' in try_name:
+                return True
+            if 'raptor-speedometer' in try_name \
+                    and 'power' in try_name \
+                    and 'fenix' in try_name:
+                return True
+
+        # Select browsertime tasks
+        if 'browsertime' in try_name and 'shippable' in platform:
+            if 'speedometer' in try_name:
+                return True
+            if '-live' in try_name:
+                # We only want to select those which should run 3 times
+                # a week here, other live site tests should be removed
+                return _run_live_site()
+            return False
+
+        # Run the following tests on android geckoview
+        if platform and 'android' not in platform:
+            return False
+        if 'geckoview' not in try_name:
+            return False
+
+        # Run cpu+memory, and power tests
+        cpu_n_memory_task = '-cpu' in try_name and '-memory' in try_name
+        power_task = '-power' in try_name
+        # Ignore cpu+memory+power tests
+        if power_task and cpu_n_memory_task:
+            return False
+        if power_task or cpu_n_memory_task:
+            if 'shippable' not in platform:
+                return False
+            if '-speedometer-' in try_name:
+                return True
+            if '-scn' in try_name and '-idle' in try_name:
+                return True
+        return False
+
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 def make_desktop_nightly_filter(platforms):
@@ -506,10 +678,7 @@ def make_desktop_nightly_filter(platforms):
         return all([
             filter_on_platforms(task, platforms),
             filter_for_project(task, parameters),
-            any([
-                task.attributes.get('nightly', False),
-                task.attributes.get('shippable', False),
-            ]),
+            task.attributes.get('shippable', False),
             # Tests and nightly only builds don't have `shipping_product` set
             task.attributes.get('shipping_product') in {None, "firefox", "thunderbird"},
             task.kind not in {'l10n'},  # no on-change l10n
@@ -523,9 +692,9 @@ def target_tasks_nightly_linux(full_task_graph, parameters, graph_config):
     nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
     filter = make_desktop_nightly_filter({
-        'linux64-nightly', 'linux-nightly', 'linux64-shippable', 'linux-shippable'
+        'linux64-shippable', 'linux-shippable'
         })
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('nightly_macosx')
@@ -533,8 +702,8 @@ def target_tasks_nightly_macosx(full_task_graph, parameters, graph_config):
     """Select the set of tasks required for a nightly build of macosx. The
     nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
-    filter = make_desktop_nightly_filter({'macosx64-nightly', 'macosx64-shippable'})
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    filter = make_desktop_nightly_filter({'macosx64-shippable'})
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('nightly_win32')
@@ -542,8 +711,8 @@ def target_tasks_nightly_win32(full_task_graph, parameters, graph_config):
     """Select the set of tasks required for a nightly build of win32 and win64.
     The nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
-    filter = make_desktop_nightly_filter({'win32-nightly', 'win32-shippable'})
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    filter = make_desktop_nightly_filter({'win32-shippable'})
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('nightly_win64')
@@ -551,8 +720,8 @@ def target_tasks_nightly_win64(full_task_graph, parameters, graph_config):
     """Select the set of tasks required for a nightly build of win32 and win64.
     The nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
-    filter = make_desktop_nightly_filter({'win64-nightly', 'win64-shippable'})
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    filter = make_desktop_nightly_filter({'win64-shippable'})
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('nightly_win64_aarch64')
@@ -560,8 +729,8 @@ def target_tasks_nightly_win64_aarch64(full_task_graph, parameters, graph_config
     """Select the set of tasks required for a nightly build of win32 and win64.
     The nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
-    filter = make_desktop_nightly_filter({'win64-aarch64-nightly', 'win64-aarch64-shippable'})
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    filter = make_desktop_nightly_filter({'win64-aarch64-shippable'})
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('nightly_asan')
@@ -570,10 +739,10 @@ def target_tasks_nightly_asan(full_task_graph, parameters, graph_config):
     nightly build process involves a pipeline of builds, signing,
     and, eventually, uploading the tasks to balrog."""
     filter = make_desktop_nightly_filter({
-        'linux64-asan-reporter-nightly',
-        'win64-asan-reporter-nightly'
+        'linux64-asan-reporter-shippable',
+        'win64-asan-reporter-shippable'
     })
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t, parameters)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t, parameters)]
 
 
 @_target_task('daily_releases')
@@ -585,7 +754,7 @@ def target_tasks_daily_releases(full_task_graph, parameters, graph_config):
     def filter(task):
         return task.kind in ['maybe-release']
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('nightly_desktop')
@@ -595,7 +764,7 @@ def target_tasks_nightly_desktop(full_task_graph, parameters, graph_config):
     # Tasks that aren't platform specific
     release_filter = make_desktop_nightly_filter({None})
     release_tasks = [
-        l for l, t in full_task_graph.tasks.iteritems()
+        l for l, t in six.iteritems(full_task_graph.tasks)
         if release_filter(t, parameters)
     ]
     # Avoid duplicate tasks.
@@ -636,14 +805,14 @@ def target_tasks_chromium_update(full_task_graph, parameters, graph_config):
             'fetch-mac-chromium']
 
 
-@_target_task('pipfile_update')
-def target_tasks_pipfile_update(full_task_graph, parameters, graph_config):
+@_target_task('python_dependency_update')
+def target_tasks_python_update(full_task_graph, parameters, graph_config):
     """Select the set of tasks required to perform nightly in-tree pipfile updates
     """
     def filter(task):
         # For now any task in the repo-update kind is ok
-        return task.kind in ['pipfile-update']
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+        return task.kind in ['python-dependency-update']
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('file_update')
@@ -653,7 +822,7 @@ def target_tasks_file_update(full_task_graph, parameters, graph_config):
     def filter(task):
         # For now any task in the repo-update kind is ok
         return task.kind in ['repo-update']
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('l10n_bump')
@@ -663,7 +832,17 @@ def target_tasks_l10n_bump(full_task_graph, parameters, graph_config):
     def filter(task):
         # For now any task in the repo-update kind is ok
         return task.kind in ['l10n-bump']
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('merge_automation')
+def target_tasks_merge_automation(full_task_graph, parameters, graph_config):
+    """Select the set of tasks required to perform repository merges.
+    """
+    def filter(task):
+        # For now any task in the repo-update kind is ok
+        return task.kind in ['merge-automation']
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('cron_bouncer_check')
@@ -675,7 +854,7 @@ def target_tasks_bouncer_check(full_task_graph, parameters, graph_config):
             return False
         # For now any task in the repo-update kind is ok
         return task.kind in ['cron-bouncer-check']
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('staging_release_builds')
@@ -697,7 +876,7 @@ def target_tasks_staging_release(full_task_graph, parameters, graph_config):
             return True
         return False
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('release_simulation')
@@ -709,7 +888,7 @@ def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
         'nightly': 'mozilla-central',
         'beta': 'mozilla-beta',
         'release': 'mozilla-release',
-        'esr60': 'mozilla-esr60',
+        'esr68': 'mozilla-esr68',
     }
     target_project = project_by_release.get(parameters['release_type'])
     if target_project is None:
@@ -726,7 +905,7 @@ def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
             return False
         return True
 
-    return [l for l, t in full_task_graph.tasks.iteritems()
+    return [l for l, t in six.iteritems(full_task_graph.tasks)
             if filter_release_tasks(t, parameters)
             and filter_out_cron(t, parameters)
             and filter_for_target_project(t)
@@ -748,10 +927,68 @@ def target_tasks_codereview(full_task_graph, parameters, graph_config):
 
         return False
 
-    return [l for l, t in full_task_graph.tasks.iteritems() if filter(t)]
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
 
 
 @_target_task('nothing')
 def target_tasks_nothing(full_task_graph, parameters, graph_config):
     """Select nothing, for DONTBUILD pushes"""
     return []
+
+
+@_target_task('raptor_tp6m')
+def target_tasks_raptor_tp6m(full_task_graph, parameters, graph_config):
+    """
+    Select tasks required for running raptor cold page-load tests on fenix and refbrow
+    """
+    def filter(task):
+        platform = task.attributes.get('build_platform')
+        attributes = task.attributes
+
+        if platform and 'android' not in platform:
+            return False
+        if attributes.get('unittest_suite') != 'raptor':
+            return False
+        try_name = attributes.get('raptor_try_name')
+        if '-cold' in try_name and 'shippable' in platform:
+            if '-1-refbrow-' in try_name:
+                return True
+            # Get browsertime amazon smoke tests
+            if 'browsertime' in try_name and \
+               'amazon' in try_name and 'search' not in try_name and \
+               'fenix' in try_name:
+                return True
+
+    return [l for l, t in six.iteritems(full_task_graph.tasks) if filter(t)]
+
+
+@_target_task('condprof')
+def target_tasks_condprof(full_task_graph, parameters, graph_config):
+    """
+    Select tasks required for building conditioned profiles.
+    """
+    for name, task in six.iteritems(full_task_graph.tasks):
+        if task.kind == "condprof":
+            yield name
+
+
+@_target_task('system_symbols')
+def target_tasks_system_symbols(full_task_graph, parameters, graph_config):
+    """
+    Select tasks for uploading system-symbols.
+    """
+    for name, task in six.iteritems(full_task_graph.tasks):
+        if task.kind == "system-symbols-upload":
+            yield name
+
+
+@_target_task('perftest')
+def target_tasks_perftest(full_task_graph, parameters, graph_config):
+    """
+    Select perftest tasks we want to run daily
+    """
+    for name, task in six.iteritems(full_task_graph.tasks):
+        if task.kind != "perftest":
+            continue
+        if task.attributes.get('cron', False):
+            yield name
